@@ -105,15 +105,6 @@ Examples
 
 """
 
-__revision__ = filter(str.isdigit, "$Revision: 117 $")
-
-__author__ = "Daniele Bigoni"
-__copyright__ = """Copyright 2012, Daniele Bigoni"""
-__credits__ = ["Daniele Bigoni"]
-__maintainer__ = "Daniele Bigoni"
-__email__ = "dabi@imm.dtu.dk"
-__status__ = "Production"
-
 import sys
 import numpy as np
 from numpy import linalg as LA
@@ -122,6 +113,7 @@ from numpy import fft as FFT
 from scipy.special import gamma as gammaF
 from scipy.misc import factorial
 from scipy.misc import comb as SPcomb
+from scipy import sparse as scsp
 
 JACOBI = 'Jacobi'
 HERMITEP = 'HermiteP'
@@ -2065,6 +2057,28 @@ def LinearShapeFunction(x,xm,xp,xi):
     if x != xp: N += ((x - xi)/(xp - x) + 1.) * ((xi >= x)*(xi <= xp)).astype(float)
     return N
 
+def SparseLinearShapeFunction(x,xm,xp,xi):
+    """ Hat function used for linear interpolation. 
+    Returns sparse indices for construction of scipy.sparse.coo_matrix.
+    
+    :param array x: 1d original points
+    :param float xm,xp: bounding points of the support of the shape function
+    :param array xi: 1d interpolation points
+
+    :returns tuple (idxs,vals): List of indexes and evaluation of the shape function on xi
+    """
+    idxs = []
+    vals = []
+    if x != xm:
+        bool_idxs = (xi >= xm)*(xi <= x)
+        idxs.extend( np.where(bool_idxs)[0] )
+        vals.extend( (xi[bool_idxs] - xm)/(x - xm) )
+    if x != xp:
+        bool_idxs = (xi >= x)*(xi <= xp)
+        idxs.extend( np.where(bool_idxs)[0] )
+        vals.extend( ((x - xi[bool_idxs])/(xp - x) + 1.) )
+    return (idxs,vals)
+
 def LinearInterpolationMatrix(x, xi):
     """
     LinearInterpolationMatrix(): constructs the Linear Interpolation Matrix from points ``x`` to points ``xi``
@@ -2089,3 +2103,43 @@ def LinearInterpolationMatrix(x, xi):
         M[:,i] = LinearShapeFunction(x[i],x[i-1],x[i+1],xi)
 
     return M
+
+def SparseLinearInterpolationMatrix(x,xi):
+    """
+    LinearInterpolationMatrix(): constructs the Linear Interpolation Matrix from points ``x`` to points ``xi``.
+    Returns a scipy.sparse.coo_matrix
+
+    Syntax:
+        ``T = LagrangeInterpolationMatrix(x, xi)``
+
+    Input:
+        * ``x`` = (1d-array,float) set of ``N`` original points
+        * ``xi`` = (1d-array,float) set of ``M`` interpolating points
+
+    Output:
+        * ``T`` = (scipy.sparse.coo_matrix(``MxN``),float) Linear Interpolation Matrix
+
+    """
+    
+    rows = []
+    cols = []
+    vals = []
+    
+    (ii,vv) = SparseLinearShapeFunction(x[0],x[0],x[1],xi)
+    rows.extend( ii )
+    cols.extend( [0] * len(ii) )
+    vals.extend( vv )
+
+    (ii,vv) = SparseLinearShapeFunction(x[-1],x[-2],x[-1],xi)
+    rows.extend( ii )
+    cols.extend( [len(x)-1] * len(ii) )
+    vals.extend( vv )
+
+    for j in range(1,len(x)-1):
+        (ii,vv) = SparseLinearShapeFunction(x[j],x[j-1],x[j+1],xi)
+        rows.extend( ii )
+        cols.extend( [j] * len(ii) )
+        vals.extend( vv )
+
+    M = scsp.coo_matrix( (np.asarray(vals), (np.asarray(rows),np.asarray(cols))), shape=( len(xi), len(x) ) )
+    return M
